using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace KeyboardMouseButtons
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            _hookID = SetHook(_proc);
            // this.Text = ((int)Keys.Return).ToString();
        }

        #region keyboard action

        // See Stephen Toub blog
        // http://blogs.msdn.com/toub/archive/2006/05/03/589423.aspx

        private const int WH_KEYBOARD_LL = 13;
        private const int WM_KEYDOWN = 0x0100;
        private const int WM_KEYUP = 0x0101;
        private const int WM_SYSKEYDOWN = 0x0104; 

        private static LowLevelKeyboardProc _proc = HookCallback;
        private static IntPtr _hookID = IntPtr.Zero;

        private static Keys _lastKey;
        private static Keys _currentKey;

        private static IntPtr SetHook(LowLevelKeyboardProc proc)
        {
            using (Process curProcess = Process.GetCurrentProcess())
            using (ProcessModule curModule = curProcess.MainModule)
            {
                return SetWindowsHookEx(WH_KEYBOARD_LL, proc,
                    GetModuleHandle(curModule.ModuleName), 0);
            }
        }

        private delegate IntPtr LowLevelKeyboardProc(int nCode, IntPtr wParam, IntPtr lParam);

        private static IntPtr HookCallback(int nCode, IntPtr wParam, IntPtr lParam)
        {
            if (nCode >= 0)
            {
                int vkCode = Marshal.ReadInt32(lParam);

                Boolean isDown = (wParam == (IntPtr)WM_KEYDOWN || wParam == (IntPtr)WM_SYSKEYDOWN);

                _lastKey = _currentKey;
                _currentKey = (Keys)vkCode;

                if (_currentKey == Keys.RWin)
                {
                    if (!(_currentKey == _lastKey && isDown)) DoClick(Cursor.Position, 1, isDown);
                    if (! isDown) _currentKey = Keys.None;
                    return new IntPtr(1);
                }
                else if (_currentKey == Keys.RMenu)
                {
                    if (!(_currentKey == _lastKey && isDown)) DoClick(Cursor.Position, 3, isDown);
                    if (!isDown) _currentKey = Keys.None;
                    return new IntPtr(1);
                }
            }

            return CallNextHookEx(_hookID, nCode, wParam, lParam);
        }

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr SetWindowsHookEx(int idHook,
            LowLevelKeyboardProc lpfn, IntPtr hMod, uint dwThreadId);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool UnhookWindowsHookEx(IntPtr hhk);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr CallNextHookEx(IntPtr hhk, int nCode,
            IntPtr wParam, IntPtr lParam);

        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr GetModuleHandle(string lpModuleName);

        #endregion


        #region mouse action

        private const UInt32 MOUSEEVENTF_LEFTDOWN = 0x0002;
        private const UInt32 MOUSEEVENTF_LEFTUP = 0x0004;
        private const UInt32 MOUSEEVENTF_RIGHTDOWN = 0x0008;
        private const UInt32 MOUSEEVENTF_RIGHTUP = 0x0010;

        [DllImport("user32.dll")]
        private static extern void mouse_event(
        UInt32 dwFlags, // motion and click options
        UInt32 dx, // horizontal position or change
        UInt32 dy, // vertical position or change
        UInt32 dwData, // wheel movement
        IntPtr dwExtraInfo // application-defined information
        );

        public static void DoClick(Point location, int buttonNum, Boolean doDown)
        {
            Cursor.Position = location;

            if (buttonNum == 1)
            {
                if (doDown)
                    mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, new System.IntPtr());
                else
                    mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, new System.IntPtr());
            }
            else if (buttonNum == 3)
            {
                if (doDown)
                    mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, new System.IntPtr());
                else
                    mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, new System.IntPtr());
            }
        }
        #endregion


        #region form action

        private void onNotifyIconDoubleClick(object sender, MouseEventArgs e)
        {
            if (this.Visible)
                this.Visible = false;
            else
                this.Visible = true;
        }

        private void onMinimizeButtonClick(object sender, EventArgs e)
        {
            this.Visible = false;
        }

        private void onQuitButtonClick(object sender, EventArgs e)
        {
            UnhookWindowsHookEx(_hookID);
            this.Close();
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.zeropointnine.com/blog/");
        }

        #endregion

        private void label1_Click(object sender, EventArgs e)
        {

        }
    }
}
